@@ -2,14 +2,13 @@ import rpy2.robjects as ro
 import rpy2.robjects.conversion as conversion
 import rpy2.rinterface as rinterface
 from rpy2.rinterface import (Sexp,
-                             SexpVector,
                              ListSexpVector,
                              StrSexpVector, ByteSexpVector,
-                             LGLSXP, INTSXP, REALSXP, CPLXSXP,
-                             STRSXP, VECSXP, NULL)
+                             RTYPES)
 import numpy
 
-#from rpy2.robjects.vectors import DataFrame, Vector, ListVector
+# TODO: move this to rinterface.
+RINT_SIZE = 32
 
 original_converter = None
 
@@ -17,119 +16,134 @@ original_converter = None
 #   http://numpy.scipy.org/array_interface.shtml
 _kinds = {
     # "t" -> not really supported by numpy
-    "b": rinterface.LGLSXP,
-    "i": rinterface.INTSXP,
+    'b': ro.vectors.BoolVector,
+    'i': ro.vectors.IntVector,
     # "u" -> special-cased below
-    "f": rinterface.REALSXP,
-    "c": rinterface.CPLXSXP,
+    'f': ro.vectors.FloatVector,
+    'c': ro.vectors.ComplexVector,
     # "O" -> special-cased below
-    "S": rinterface.STRSXP,
-    "U": rinterface.STRSXP,
+    'S': ro.vectors.ByteVector,
+    'U': ro.vectors.StrVector,
     # "V" -> special-cased below
-    #FIXME: datetime64 ?
-    #"datetime64":
+    # TODO: datetime64 ?
+    # "datetime64":
     }
 
-#FIXME: the following would need further thinking & testing on
-#       32bits architectures
-_kinds['float64'] = rinterface.REALSXP
 
-_vectortypes = (rinterface.LGLSXP,
-                rinterface.INTSXP,
-                rinterface.REALSXP,
-                rinterface.CPLXSXP,
-                rinterface.STRSXP)
+_vectortypes = (RTYPES.LGLSXP,
+                RTYPES.INTSXP,
+                RTYPES.REALSXP,
+                RTYPES.CPLXSXP,
+                RTYPES.STRSXP)
 
 converter = conversion.Converter('original numpy conversion')
-py2ri = converter.py2ri
-py2ro = converter.py2ro
-ri2py = converter.ri2py
-ri2ro = converter.ri2ro
+py2rpy = converter.py2rpy
+rpy2py = converter.rpy2py
 
-import sys
 
-if sys.version_info[0] == 3:
-    def numpy_O_py2ri(o):
-        if all(isinstance(x, str) for x in o):
-            res = StrSexpVector(o)
-        elif all(isinstance(x, bytes) for x in o):
-            res = ByteSexpVector(o)
-        else:
-            res = conversion.py2ri(list(o))
-        return res
-else:
-    def numpy_O_py2ri(o):
-        if all((isinstance(x, str) or isinstance(x, bytes) or isinstance(x, unicode)) for x in o):
-            res = StrSexpVector(o)
-        else:
-            res = conversion.py2ri(list(o))
-        return res
-    
-@py2ri.register(numpy.ndarray)
-def numpy2ri(o):
+def numpy_O_py2rpy(o):
+    if all(isinstance(x, str) for x in o):
+        res = StrSexpVector(o)
+    elif all(isinstance(x, bytes) for x in o):
+        res = ByteSexpVector(o)
+    else:
+        res = conversion.py2rpy(list(o))
+    return res
+
+
+def _numpyarray_to_r(a, func):
+    # "F" means "use column-major order"
+    vec = func(numpy.ravel(a, order='F'))
+    # TODO: no dimnames ?
+    # TODO: optimize what is below needed/possible ?
+    #  (other ways to create R arrays ?)
+    dim = ro.vectors.IntVector(a.shape)
+    res = rinterface.baseenv['array'](vec, dim=dim)
+    return res
+
+
+def unsignednumpyint_to_rint(intarray):
+    """Convert a numpy array of unsigned integers to an R array."""
+    if intarray.itemsize >= (RINT_SIZE / 8):
+        raise ValueError(
+            'Cannot convert numpy array of {numpy_type!s} '
+            '(R integers are signed {RINT_SIZE}-bit integers).'
+            .format(numpy_type=intarray.dtype.type,
+                    RINT_SIZE=RINT_SIZE)
+        )
+    else:
+        res = _numpyarray_to_r(intarray, _kinds['i'])
+    return res
+
+
+@py2rpy.register(numpy.ndarray)
+def numpy2rpy(o):
     """ Augmented conversion function, converting numpy arrays into
     rpy2.rinterface-level R structures. """
     if not o.dtype.isnative:
-        raise(ValueError("Cannot pass numpy arrays with non-native byte orders at the moment."))
+        raise ValueError('Cannot pass numpy arrays with non-native '
+                         'byte orders at the moment.')
 
     # Most types map onto R arrays:
     if o.dtype.kind in _kinds:
-        # "F" means "use column-major order"
-        vec = SexpVector(o.ravel("F"), _kinds[o.dtype.kind])
-        dim = SexpVector(o.shape, INTSXP)
-        #FIXME: no dimnames ?
-        #FIXME: optimize what is below needed/possible ? (other ways to create R arrays ?)
-        res = rinterface.baseenv['array'](vec, dim=dim)
+        res = _numpyarray_to_r(o, _kinds[o.dtype.kind])
     # R does not support unsigned types:
-    elif o.dtype.kind == "u":
-        raise(ValueError("Cannot convert numpy array of unsigned values -- R does not have unsigned integers."))
+    elif o.dtype.kind == 'u':
+        res = unsignednumpyint_to_rint(o)
     # Array-of-PyObject is treated like a Python list:
-    elif o.dtype.kind == "O":
-        res = numpy_O_py2ri(o)
+    elif o.dtype.kind == 'O':
+        res = numpy_O_py2rpy(o)
     # Record arrays map onto R data frames:
-    elif o.dtype.kind == "V":
+    elif o.dtype.kind == 'V':
         if o.dtype.names is None:
-            raise(ValueError("Nothing can be done for this numpy array type %s at the moment." % (o.dtype,)))
+            raise ValueError('Nothing can be done for this numpy array '
+                             'type "%s" at the moment.' % (o.dtype,))
         df_args = []
         for field_name in o.dtype.names:
             df_args.append((field_name,
-                            conversion.py2ri(o[field_name])))
+                            conversion.py2rpy(o[field_name])))
         res = ro.baseenv["data.frame"].rcall(tuple(df_args), ro.globalenv)
     # It should be impossible to get here:
     else:
-        raise(ValueError("Unknown numpy array type '%s'." % str(o.dtype)))
+        raise ValueError('Unknown numpy array type "%s".' % str(o.dtype))
     return res
 
-@py2ri.register(numpy.integer)
-def npint_py2ri(obj):
-    return ro.int2ri(obj)
 
-@py2ri.register(numpy.floating)
-def npfloat_py2ri(obj):
-    return rinterface.SexpVector([obj, ], rinterface.REALSXP)
+@py2rpy.register(numpy.integer)
+def npint_py2rpy(obj):
+    return rinterface.IntSexpVector([obj, ])
 
-@py2ri.register(object)
-def nonnumpy2ri(obj):
+
+@py2rpy.register(numpy.floating)
+def npfloat_py2rpy(obj):
+    return rinterface.FloatSexpVector([obj, ])
+
+
+@py2rpy.register(object)
+def nonnumpy2rpy(obj):
     # allow array-like objects to also function with this module.
     if not isinstance(obj, numpy.ndarray) and hasattr(obj, '__array__'):
         obj = obj.__array__()
-        return ro.default_converter.py2ri(obj)
+        return ro.default_converter.py2rpy(obj)
     elif original_converter is None:
         # This means that the conversion module was not "activated".
         # For now, go with the default_converter.
         # TODO: the conversion system needs an overhaul badly.
-        return ro.default_converter.py2ri(obj)
+        return ro.default_converter.py2rpy(obj)
     else:
         # The conversion module was "activated"
-        return original_converter.py2ri(obj)
+        return original_converter.py2rpy(obj)
 
-@py2ro.register(numpy.ndarray)
-def numpy2ro(obj):
-    res = numpy2ri(obj)
-    return ro.vectors.rtypeof2rotype[res.typeof](res)
 
-@ri2py.register(ListSexpVector)
-def ri2py_list(obj):
+# TODO: delete ?
+# @py2ro.register(numpy.ndarray)
+# def numpy2ro(obj):
+#     res = numpy2ri(obj)
+#     return ro.vectors.rtypeof2rotype[res.typeof](res)
+
+
+@rpy2py.register(ListSexpVector)
+def rpy2py_list(obj):
     if 'data.frame' in obj.rclass:
         # R "factor" vectors will not convert well by default
         # (will become integers), so we build a temporary list o2
@@ -144,23 +158,25 @@ def ri2py_list(obj):
                 column = tuple(levels[x-1] for x in column)
             o2.append(column)
         names = obj.do_slot('names')
-        if names is NULL:
+        if names == rinterface.NULL:
             res = numpy.rec.fromarrays(o2)
         else:
             res = numpy.rec.fromarrays(o2, names=tuple(names))
     else:
         # not a data.frame, yet is it still possible to convert it
-        res = ro.default_converter.ri2py(obj)
+        res = ro.default_converter.rpy2py(obj)
     return res
 
-@ri2py.register(Sexp)
-def ri2py_sexp(obj):
-    if (obj.typeof in _vectortypes) and (obj.typeof != VECSXP):
-        res = numpy.asarray(obj)
+
+@rpy2py.register(Sexp)
+def rpy2py_sexp(obj):
+    if (obj.typeof in _vectortypes) and (obj.typeof != RTYPES.VECSXP):
+        res = numpy.array(obj)
     else:
-        res = ro.default_converter.ri2py(obj)
+        res = ro.default_converter.rpy2py(obj)
     return res
 
+
 def activate():
     global original_converter
 
@@ -172,25 +188,15 @@ def activate():
     new_converter = conversion.Converter('numpy conversion',
                                          template=original_converter)
 
-    for k,v in py2ri.registry.items():
+    for k, v in py2rpy.registry.items():
         if k is object:
             continue
-        new_converter.py2ri.register(k, v)
+        new_converter.py2rpy.register(k, v)
 
-    for k,v in ri2ro.registry.items():
+    for k, v in rpy2py.registry.items():
         if k is object:
             continue
-        new_converter.ri2ro.register(k, v)
-
-    for k,v in py2ro.registry.items():
-        if k is object:
-            continue
-        new_converter.py2ro.register(k, v)
-
-    for k,v in ri2py.registry.items():
-        if k is object:
-            continue
-        new_converter.ri2py.register(k, v)
+        new_converter.rpy2py.register(k, v)
 
     conversion.set_conversion(new_converter)
 
